package model;

public class Board {

	public static final int PAWN = 0, ROOK = 1, KNIGHT = 2, BISHOP = 3, QUEEN = 4, KING = 5;

	private Piece[] pieces;
	private Position selected;

	public Board() {
		selected = null;
		pieces = new Piece[64];
		// initialisation of the list of pieces
		for(int i = 0 ; i < 64 ; i++) {
			pieces[i] = null;
		}
		int y = 0;
		for(int color = Piece.WHITE ; color == Piece.WHITE || color == Piece.BLACK ; color++) {
			// placement of pawns
			if(color == Piece.WHITE) { y = 2; }
			else { y = 7; }
			for(int x = Position.a ; x <= Position.h ; x++) {
				createPiece(x, y, color, PAWN); // fills line 2 and line 7 with pawns
			}
			// placement of rook, knight, bishop, queen, king - line 1 and line 8
			if(color == Piece.WHITE) { y = 1; }
			else { y = 8; }
			createPiece(Position.a, y, color, ROOK);
			createPiece(Position.b, y, color, KNIGHT);
			createPiece(Position.c, y, color, BISHOP);
			createPiece(Position.d, y, color, QUEEN);
			createPiece(Position.e, y, color, KING);
			createPiece(Position.f, y, color, BISHOP);
			createPiece(Position.g, y, color, KNIGHT);
			createPiece(Position.h, y, color, ROOK);
		}
	}

	public void setPiece(Piece piece) {
		pieces[piece.getPosition().getID()] = piece;
	}

	public Piece getPiece(Position position) {
		return pieces[position.getID()];
	}
	public void createPiece(int x, int y, int color, int type) {
		switch(type) {
			case ROOK -> setPiece(new Rook(new Position(x, y), color));
			case KNIGHT -> setPiece(new Knight(new Position(x, y), color));
			case BISHOP -> setPiece(new Bishop(new Position(x, y), color));
			case QUEEN -> setPiece(new Queen(new Position(x, y), color));
			case KING -> setPiece(new King(new Position(x, y), color));
			default -> setPiece(new Pawn(new Position(x, y), color));
		}
	}

	public Position getSelected() {
		return selected;
	}
	public Piece getSelectedPiece() {
		return getPiece(selected);
	}

	public void setSelected(Position p) {
		selected = p;
	}

	public void movePiece(Position origin, Position arrival) {
		Piece piece = getPiece(origin);
		piece.setPosition(arrival); // changes position to arrival
		setPiece(piece); // copies the piece from origin ID to arrival ID
		pieces[origin.getID()] = null; // removes the origin ID piece
	}

	public boolean isFree(Position p) {
		return (Position.isCorrect(p) && pieces[p.getID()] == null);
	}
	public boolean isBlack(Position p) {
		return (!isFree(p) && pieces[p.getID()].getColor() == Piece.BLACK);
	}

	public boolean isWhite(Position p) {
		return (!isFree(p) && pieces[p.getID()].getColor() == Piece.WHITE);
	}

	public boolean isEnnemy(Position p, Position origin) {
		return (Position.isCorrect(p) &&  !isFree(p) && pieces[p.getID()].getColor() != pieces[origin.getID()].getColor() );
	}
	public boolean isKing(Position p) {
		return (!isFree(p) && pieces[p.getID()].getClass() == King.class);
	}

	public boolean isPawn(Position p) {
		return (!isFree(p) && pieces[p.getID()].getClass() == Pawn.class);
	}
}
