package view;

import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.scene.Group;
import javafx.scene.PerspectiveCamera;
import javafx.scene.Scene;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Translate;

public class CustomCamera {

	private PerspectiveCamera camera;
	private Group group; // used to rotate camera from 0,0,0 and not from camera x,y,z
	private double anchorX, anchorY, anchorAngleX, anchorAngleY;

	private final static double SCROLL_SPEED = 10;
	private DoubleProperty angleX, angleY;
	public CustomCamera(int nearClip, int farClip, int fov) {
		camera = new PerspectiveCamera(true);
		camera.setFieldOfView(fov);
		camera.getTransforms().add(new Translate(0,0,-130));
		camera.setNearClip(nearClip);
		camera.setFarClip(farClip);
		group = new Group(camera);
		group.setTranslateY(20);
		// camera controls for user (with mouse)
		angleX = new SimpleDoubleProperty(-45);
		angleY = new SimpleDoubleProperty(8);
	}

	public Group getGroup() {
		return group;
	}

	public PerspectiveCamera getCamera() {
		return camera;
	}

	public void initMouseControl(Scene scene) { // allows user to move camera with mouse, with some constraints
		Rotate xRotate;
		Rotate yRotate;
		group.getTransforms().addAll(
				yRotate = new Rotate(0, Rotate.Y_AXIS),
				xRotate = new Rotate(-25, Rotate.X_AXIS)
		);
		xRotate.angleProperty().bind(angleX);
		yRotate.angleProperty().bind(angleY);

		scene.setOnMousePressed(event -> {
			anchorX = event.getSceneX();
			anchorY = event.getSceneY();
			anchorAngleX = angleX.get();
			anchorAngleY = angleY.get();

		});

		int min = -90;
		int max = -18;

		scene.setOnMouseDragged(event -> {
			angleX.set(anchorAngleX + (anchorY - event.getSceneY()));
			angleY.set(anchorAngleY - (anchorX - event.getSceneX()));
			if(angleX.get() > max) {
				angleX.set(max);
				anchorY = event.getSceneY();
				anchorAngleX = angleX.get();
			}
			if(angleX.get() < min) {
				angleX.set(min);
				anchorY = event.getSceneY();
				anchorAngleX = angleX.get();
			}
		});
	}

	public void scrollCamera(double deltaY) {
		double z = camera.getTranslateZ();
		if(deltaY > 0) {
			if(z < 30)
				camera.setTranslateZ(z + SCROLL_SPEED);
		}
		else {
			if(z > -400)
				camera.setTranslateZ(z - SCROLL_SPEED);
		}
	}
}
